@@ -1,8 +1,7 @@
-/*  $Id: QuadruplebackEngine.cpp,v 1.2 2017/10/09 18:48:55 sarrazip Exp $
-    QuadruplebackEngine.cpp - Game engine.
+/*  QuadruplebackEngine.cpp - Game engine.
 
     quadrupleback - A video game where intruders must be circled.
-    Copyright (C) 2012 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2012-2024 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -26,15 +25,17 @@
 
 #include "Enemy.h"
 
+// Some XPM images are compiled as C, in addition to be loaded by the
+// QuadruplebackEngine constructor, so that the C structures can be used
+// for collision detection.
+//
 #include "images/cherry.xpm"
 #include "images/apple.xpm"
 #include "images/pear.xpm"
 #include "images/horseshoe.xpm"
 #include "images/skull.xpm"
 #include "images/skate0.xpm"
-#include "images/skate1.xpm"
 #include "images/spider0.xpm"
-#include "images/spider1.xpm"
 #include "images/yoyo.xpm"
 
 #include <fstream>
@@ -56,8 +57,10 @@ using namespace flatzebra;
 ///////////////////////////////////////////////////////////////////////////////
 
 
-QuadruplebackEngine::QuadruplebackEngine(const string &windowManagerCaption, bool _useSound)
-  : GameEngine(Couple(900, 675), windowManagerCaption, false, true),
+QuadruplebackEngine::QuadruplebackEngine(const string &windowManagerCaption,
+                                         bool _useSound,
+                                         bool _useAcceleratedRendering)
+  : GameEngine(Couple(900, 675), windowManagerCaption, false, true, _useAcceleratedRendering),
                   // may throw string exception
     paused(false),
     tickCount(0),
@@ -77,31 +80,31 @@ QuadruplebackEngine::QuadruplebackEngine(const string &windowManagerCaption, boo
               MARGIN,
               theScreenSizeInPixels.x - 2 * MARGIN,
               CHAR_HEIGHT,
-              mapColor(  0, 128, 255),
-              mapColor(255, 255, 255)),
+              mapRGBA(  0, 128, 255),
+              mapRGBA(255, 255, 255)),
     scoreArea(MARGIN,
               theScreenSizeInPixels.y - MARGIN - CHAR_HEIGHT,  // at bottom
               theScreenSizeInPixels.x - 2 * MARGIN,
               CHAR_HEIGHT,
-              mapColor(  0, 128, 255),
-              mapColor(255, 255, 255)),
+              mapRGBA(  0, 128, 255),
+              mapRGBA(255, 255, 255)),
     gameArea( MARGIN,
               MARGIN + livesArea.size.y,
               theScreenSizeInPixels.x - 2 * MARGIN,
               theScreenSizeInPixels.y - 2 * MARGIN - livesArea.size.y - scoreArea.size.y,
-              mapColor(  0,   0,   0),  // unused
-              mapColor(  0,   0,   0)),
-    presentationColor(  mapColor(255, 255, 255)),
-    errorMessageColor(  mapColor(255,   0,   0)),
-    pauseBGColor(       mapColor( 64,  64,  64)),
-    snakeColor(         mapColor(  0, 128, 255)),
-    loopColor(          mapColor(255,   0, 255)),
-    intersectionColor(  mapColor(255, 255,   0)),
-    currentLifeColor(   mapColor(255, 128,   0)),
-    otherLivesColor(    mapColor(0,   128, 255)),
-    floatingNumberColor(mapColor(255, 128,   0)),
-    yoyoWireColor(      mapColor(255, 128,   0)),
-    deathColor(         mapColor(  0, 255,   0, 128)),
+              mapRGBA(  0,   0,   0),  // unused
+              mapRGBA(  0,   0,   0)),
+    presentationColor(  mapRGBA(255, 255, 255)),
+    errorMessageColor(  mapRGBA(255,   0,   0)),
+    pauseBGColor(       mapRGBA( 64,  64,  64)),
+    snakeColor(         mapRGBA(  0, 128, 255)),
+    loopColor(          mapRGBA(255,   0, 255)),
+    intersectionColor(  mapRGBA(255, 255,   0)),
+    currentLifeColor(   mapRGBA(255, 128,   0)),
+    otherLivesColor(    mapRGBA(0,   128, 255)),
+    floatingNumberColor(mapRGBA(255, 128,   0)),
+    yoyoWireColor(      mapRGBA(255, 128,   0)),
+    deathColor(         mapRGBA(  0, 255,   0, 128)),
     joystick(),
     snake(),
     enemies(),
@@ -131,16 +134,16 @@ QuadruplebackEngine::QuadruplebackEngine(const string &windowManagerCaption, boo
     enemyCapturedSound(),
     enemyTouchedSound()
 {
-    loadPixmap(cherry_xpm, cherryPA, 0);
-    loadPixmap(apple_xpm, applePA, 0);
-    loadPixmap(pear_xpm, pearPA, 0);
-    loadPixmap(horseshoe_xpm, horseshoePA, 0);
-    loadPixmap(skull_xpm, skullPA, 0);
-    loadPixmap(skate0_xpm, skatePA, 0);
-    loadPixmap(skate1_xpm, skatePA, 1);
-    loadPixmap(spider0_xpm, spiderPA, 0);
-    loadPixmap(spider1_xpm, spiderPA, 1);
-    loadPixmap(yoyo_xpm, yoyoPA, 0);
+    loadPixmap("cherry.xpm", cherryPA, 0);
+    loadPixmap("apple.xpm", applePA, 0);
+    loadPixmap("pear.xpm", pearPA, 0);
+    loadPixmap("horseshoe.xpm", horseshoePA, 0);
+    loadPixmap("skull.xpm", skullPA, 0);
+    loadPixmap("skate0.xpm", skatePA, 0);
+    loadPixmap("skate1.xpm", skatePA, 1);
+    loadPixmap("spider0.xpm", spiderPA, 0);
+    loadPixmap("spider1.xpm", spiderPA, 1);
+    loadPixmap("yoyo.xpm", yoyoPA, 0);
 
     const char *pkgDataDir = getenv("PKGDATADIR");
     if (pkgDataDir == NULL)
@@ -167,6 +170,7 @@ QuadruplebackEngine::QuadruplebackEngine(const string &windowManagerCaption, boo
         if (file.tellg() != FILE_LEN)
             throw string("failed to read entire font definition file ") + filename;
 
+        // Set the font to be used by gfxWriteString(), i.e., SDL2_gfx's stringRGBA().
         gfxPrimitivesSetFont(fontDefBuffer.data(), 9, 18);
     }
 
@@ -221,6 +225,16 @@ QuadruplebackEngine::~QuadruplebackEngine()
 }
 
 
+void
+QuadruplebackEngine::loadPixmap(const char *filePath,
+                                PixmapArray &pa,
+                                size_t index)
+{
+    static const string dir = getDirPathFromEnv(PKGPIXMAPDIR, "PKGPIXMAPDIR");
+    GameEngine::loadPixmap(dir + filePath, pa, index);
+}
+
+
 void
 QuadruplebackEngine::resetGame()
 {
@@ -254,7 +268,7 @@ QuadruplebackEngine::initNewLife()
 
 //virtual
 void
-QuadruplebackEngine::processKey(SDLKey keysym, bool pressed)
+QuadruplebackEngine::processKey(SDL_Keycode keysym, bool pressed)
 {
     quitKS.check(keysym, pressed);
     startKS.check(keysym, pressed);
@@ -435,7 +449,7 @@ QuadruplebackEngine::tick()
     // Check for full screen toggle.
     //
     if (fullScreenKS.justPressed())
-        setVideoMode(theScreenSizeInPixels, !inFullScreenMode());  // ignore failure
+        setFullScreenMode(!inFullScreenMode());  // ignore failure
 
     // Check for pause/resume.
     //
@@ -456,12 +470,15 @@ QuadruplebackEngine::tick()
         switch (gamePhase)
         {
         case WAITING_FOR_PLAYER:
+        {
             assert(numLives == 0);
 
             moveSnakeRandomly();
 
-            if (joystick.buttonJustPressed(Joystick::START_BTN)
-                || startKS.justPressed())
+            bool startBtnJustP = joystick.buttonJustPressed(Joystick::START_BTN);
+            bool startKSJustP = startKS.justPressed();
+            if (startBtnJustP
+                || startKSJustP)
             {
                 resetGame();
                 numLives = 3;
@@ -470,6 +487,7 @@ QuadruplebackEngine::tick()
                 preLifeEndTime = tickCount + FPS * 1;
             }
             break;
+        }
         case DELAY_BEFORE_LIFE_START:
             assert(numLives > 0);
             if (tickCount >= preLifeEndTime)
@@ -757,11 +775,10 @@ QuadruplebackEngine::isPositionNearSnakeHead(const RCouple &pos, double maxDista
 
 void
 QuadruplebackEngine::drawFilledRect(const RCouple &topLeft,
-                                 const RCouple &size,
-                                 Uint32 color)
+                                    const RCouple &size,
+                                    SDL_Color color)
 {
-    RCouple botRight = topLeft + size;
-    boxColor(theSDLScreen, Sint16(topLeft.x), Sint16(topLeft.y), Sint16(botRight.x), Sint16(botRight.y), color);
+    fillRect(int(topLeft.x), int(topLeft.y), int(size.x), int(size.y), color);
 }
 
 
@@ -775,6 +792,10 @@ QuadruplebackEngine::drawAreaBG(const Area &area)
 void
 QuadruplebackEngine::drawScene()
 {
+    // Fill the whole screen in white.
+    // Most of the screen will be filled in black by gameArea.
+    // A white frame will be left around gameArea.
+    //
     drawFilledRect(RCouple(0, 0), theScreenSizeInPixels, livesArea.bgColor);
     drawAreaBG(livesArea);
     drawAreaBG(gameArea);
@@ -809,10 +830,7 @@ QuadruplebackEngine::drawScene()
             RCouple curPoint = snake.points.getPoint(i) + gameArea.topLeft;
 
             if (prevPoint != curPoint)
-                aalineColor(theSDLScreen,
-                            Sint16(prevPoint.x), Sint16(prevPoint.y),
-                            Sint16(curPoint.x), Sint16(curPoint.y), 
-                            snakeColor);
+                drawLine(int(prevPoint.x), int(prevPoint.y), int(curPoint.x), int(curPoint.y), snakeColor);
 
             prevPoint = curPoint;
         }
@@ -822,8 +840,6 @@ QuadruplebackEngine::drawScene()
                                                      it != snake.points.getIntersections().end(); ++it)
             {
                 const Intersection *inter = *it;
-                RCouple pt = inter->point + gameArea.topLeft;
-                filledCircleColor(theSDLScreen, pt.x, pt.y, 4, intersectionColor);
 
                 const vector<vector<flatzebra::RCouple> *> &loops = inter->getLoops();
                 for (vector<vector<flatzebra::RCouple> *>::const_iterator jt = loops.begin();
@@ -834,9 +850,7 @@ QuadruplebackEngine::drawScene()
 
     // Snake's head (useful if snake is entirely in one position):
     RCouple head = snake.getHeadPos() + gameArea.topLeft;
-    filledCircleColor(theSDLScreen,
-                      Sint16(head.x), Sint16(head.y),
-                      3, snakeColor);
+    fillCircle(int(head.x), int(head.y), 3, snakeColor);
 
 
     // Show number of lives left as three horizontal bars in livesArea.
@@ -879,7 +893,7 @@ QuadruplebackEngine::drawScene()
             //
             size_t widthInPixels = strlen(tmp) * CHAR_WIDTH;
             RCouple pos = fn->pos + gameArea.topLeft - RCouple(widthInPixels, CHAR_HEIGHT) / 2;
-            stringColor(theSDLScreen, pos.x, pos.y, tmp, floatingNumberColor);
+            gfxWriteString(tmp, pos.round(), floatingNumberColor);
 
             fn->pos.y -= fn->speed;
             --fn->numTicksLeft;
@@ -943,16 +957,18 @@ QuadruplebackEngine::drawScene()
     snprintf(tmp, sizeof(tmp), "%ld", score);
     writeCenteredString(scoreArea.topLeft.y, tmp, scoreArea.fgColor);
 
+    // Draw a transparent rectangle over the whole screen.
+    //
     if (gamePhase == PLAYER_DYING)
         drawFilledRect(RCouple(0, 0), theScreenSizeInPixels, deathColor);
 }
 
 
 void
-QuadruplebackEngine::writeCenteredString(Sint16 y, const char *s, Uint32 color)
+QuadruplebackEngine::writeCenteredString(Sint16 y, const char *s, SDL_Color color)
 {
-    Sint16 x = (theScreenSizeInPixels.x - strlen(s) * CHAR_WIDTH) / 2;
-    stringColor(theSDLScreen, x, y, s, color);
+    int x = (theScreenSizeInPixels.x - (int) strlen(s) * CHAR_WIDTH) / 2;
+    gfxWriteString(s, Couple(x, y), color);
 }
 
 
@@ -968,10 +984,7 @@ QuadruplebackEngine::drawLoop(const vector<RCouple> &loop)
         const RCouple &curPoint = *it + gameArea.topLeft;
 
         if (prevPoint != curPoint)
-            aalineColor(theSDLScreen,
-                        Sint16(prevPoint.x), Sint16(prevPoint.y),
-                        Sint16(curPoint.x), Sint16(curPoint.y), 
-                        loopColor);
+            drawLine(int(prevPoint.x), int(prevPoint.y), int(curPoint.x), int(curPoint.y), loopColor);
 
         prevPoint = curPoint;
     }
